<?php

namespace Drupal\celum_connect\Plugin\Field\FieldType;

use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldItemInterface;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\TypedData\DataDefinition;


/**
 * Plugin implementation of the 'Celum:connect' field type.
 *
 * @FieldType(
 *   id = "celum_connect_field",
 *   label = @Translation("Celum:connect field"),
 *   default_widget = "celum_connect_widget",
 *   default_formatter = "celum_connect_formatter"
 * )
 */
class CelumConnectFieldType extends FieldItemBase implements FieldItemInterface{


  /**
   * {@inheritdoc}
   */
  public static function schema(FieldStorageDefinitionInterface $field) {

    return [
      'columns' => [
        'id' => [
          'type' => 'int',
          'length' => 10,
          'not null' => FALSE,
        ],
        'version' => [
          'type' => 'int',
          'length' => 10,
          'not null' => FALSE,
        ],
        'downloadFormat' => [
          'type' => 'int',
          'length' => 10,
          'not null' => FALSE,
        ],
        'fileExtension' => [
          'type' => 'varchar',
          'length' => 4,
          'not null' => FALSE,
        ],
        'title' => [
          'type' => 'varchar',
          'length' => 100,
          'not null' => FALSE,
        ],
        'fileCategory' => [
          'type' => 'varchar',
          'length' => 100,
          'not null' => FALSE,
        ],
        'uri' => [
          'type' => 'varchar',
          'length' => 100,
          'not null' => FALSE,
        ],
        'thumb' => [
          'type' => 'varchar',
          'length' => 100,
          'not null' => FALSE,
        ],
        'saved' => [
          'type' => 'varchar',
          'length' => 100,
          'not null' => False,
        ],
        'type' => [
          'type' => 'varchar',
          'length' => 100,
          'not null' => False,
        ]
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function isEmpty() {
    $value = $this->get('id')->getValue();
    return $value === NULL || $value === '';
  }

  /**
   * {@inheritdoc}
   */
  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition) {
    $properties['id'] = DataDefinition::create('integer')
      ->setLabel(new TranslatableMarkup('Id'));
    $properties['version'] = DataDefinition::create('integer')
      ->setLabel(new TranslatableMarkup('Version'));
    $properties['downloadFormat'] = DataDefinition::create('integer')
      ->setLabel(new TranslatableMarkup('Downloadformat'));
    $properties['fileExtension'] = DataDefinition::create('string')
      ->setLabel(new TranslatableMarkup('File extension'));
    $properties['title'] = DataDefinition::create('string')
      ->setLabel(new TranslatableMarkup('Title'));
    $properties['fileCategory'] = DataDefinition::create('string')
      ->setLabel(new TranslatableMarkup('File category'));
    $properties['uri'] = DataDefinition::create('string')
      ->setLabel(new TranslatableMarkup('uri'));
    $properties['download'] = DataDefinition::create('string')
      ->setLabel(new TranslatableMarkup('download'));
    $properties['saved'] = DataDefinition::create('string')
      ->setLabel(new TranslatableMarkup('saved'));
    $properties['thumb'] = DataDefinition::create('string')
      ->setLabel(new TranslatableMarkup('thumb'));
    $properties['type'] = DataDefinition::create('string')
      ->setLabel(new TranslatableMarkup('type'));
    return $properties;
  }

  /**
   * {@inheritdoc}
   */
  public function setValue($values, $notify = TRUE) {
    if(isset($values['id'])){
      $config = \Drupal::service('config.factory')->getEditable('celum_connect.settings');
      if(isset($values['saved']) && $values['saved']){
        switch ($values['type']) {
          case 'direct_download_public_url_link':
            list($url, $t) = explode("_", $this->decrypt($config->get('celum_connect_licenseKey')));
            $values['uri'] = $url."/direct/download?id=".$values['id']."&format=".$values['downloadFormat']."&consumer=Drupal8";
            $values['thumb'] = $url."/direct/download?id=".$values['id']."&format=thmb&consumer=Drupal8";
            if($config->get('celum_connect_use_public_urls')) {
              $id_descriptionMap = array();
              $dlf_publicUrl_array = explode(',', $config->get('celum_connect_public_url_map'));
              foreach ($dlf_publicUrl_array as $dlf_publicUrl) {
                $dlf_publicUrl_string_array = explode(':', $dlf_publicUrl);
                $id_descriptionMap[$dlf_publicUrl_string_array[0]] = $dlf_publicUrl_string_array[1];
              }
              $publicUrlMap = $this->getPublicUrlMap($values['id'], $config);
              if($id_descriptionMap[$values['downloadFormat']] != null && $publicUrlMap[$id_descriptionMap[$values['downloadFormat']]] != null) {
                $values['uri'] = $publicUrlMap[$id_descriptionMap[$values['downloadFormat']]];
              }
            }
            break;
          case 'anura_link':
            list($url, $t) = explode("_", $this->decrypt($config->get('celum_connect_licenseKey')));
            if($config->get('celum_connect_anura_urlDispatcherMap')) {
              $url_dispatcher_array = explode(';', $config->get('celum_connect_anura_urlDispatcherMap'));
              $foundDispatcher = false;
              foreach ($url_dispatcher_array as $url_dispatcher) {
                $url_dispatcher_string_array = explode('>', $url_dispatcher);
                if(\Drupal::request()->getSchemeAndHttpHost() == $url_dispatcher_string_array[0]) {
                  $values['uri'] = $url.'/anura/'.$url_dispatcher_string_array[1].'/asset.do?preview='.$item->id.'&img=a.jpg';
                  $foundDispatcher = true;
                  break;
                }
              }
              if(!$foundDispatcher) {
                $values['uri'] = $url.'/anura/'.$config->get('celum_connect_anura_dispatcher').'/asset.do?preview='.$item->id.'&img=a.jpg';
              }
            } else {
              $values['uri'] = $url.'/anura/'.$config->get('celum_connect_anura_dispatcher').'/asset.do?preview='.$item->id.'&img=a.jpg';
            }
            $values['thumb'] = \Drupal::config('system.file')->get('default_scheme')."://celum_connect/".$values['id']."_".$values['version']."_".$values['downloadFormat'].".".$values['fileExtension']."_thumb";
            break;
          case 'anura_deep_link':
            $values['uri'] = '#asset='.$values['id'];
            $values['thumb'] = \Drupal::config('system.file')->get('default_scheme')."://celum_connect/".$values['id']."_".$values['version']."_".$values['downloadFormat'].".".$values['fileExtension']."_thumb";
            break;
          case 'download_asset':
          default:
            $values['uri'] = \Drupal::config('system.file')->get('default_scheme')."://celum_connect/".$values['id']."_".$values['version']."_".$values['downloadFormat'].".".$values['fileExtension'];
            $values['thumb'] = \Drupal::config('system.file')->get('default_scheme')."://celum_connect/".$values['id']."_".$values['version']."_".$values['downloadFormat'].".".$values['fileExtension']."_thumb";
            break;
        }
      }
    }
    parent::setValue($values, $notify);
  }

  function getPublicUrlMap($assetId, $config) {
    if(strpos($this->decrypt($config->get('celum_connect_licenseKey')), '_') !== false){
      list($publicURLRequest, $t) = explode("_", $this->decrypt($config->get('celum_connect_licenseKey')));
    }else{
      $publicURLRequest =  $this->decrypt($config->get('celum_connect_licenseKey'));
    }
    $publicURLRequest .= '/cora/Assets('.$assetId.')?$select=id&$expand=publicUrls';

    $options  = ['http' => ['header' => "Authorization: celumApiKey ".$config->get('celum_connect_apikey')]];
    $context  = stream_context_create($options);
    $publicUrlsResponse = file_get_contents($publicURLRequest, true, $context);
    $publicUrls = json_decode($publicUrlsResponse)->publicUrls;
    $publicUrlDescription_publicUrl_map = array();
    foreach( $publicUrls as $i){
      $publicUrlDescription_publicUrl_map[$i->provider.';'.$i->description] = $i->url;
    }
    return $publicUrlDescription_publicUrl_map;
  }

  /**
   * {@inheritdoc}
   */
  function preSave() {
    if(!$this->get('saved')->getValue()) {
      $this->get('saved')->setValue('true');
      $download_url = $this->get('download')->getValue();
      $thumb_url = $this->get('thumb')->getValue();
      $id = $this->get('id')->getValue();
      $version = $this->get('version')->getValue();
      $dlf = $this->get('downloadFormat')->getValue();
      $type = $this->get('type')->getValue();
      $fileExtension = $this->get('fileExtension')->getValue();
      if($download_url != NULL && $type == 'download'){
        $config = \Drupal::service('config.factory')->getEditable('celum_connect.settings');
        list($url, $t) = explode("_", $this->decrypt($config->get('celum_connect_licenseKey')));
        $download_url = $url."/direct/download?id=".$id."&format=".$dlf."&consumer=Drupal8";
        if(system_retrieve_file($download_url,
          \Drupal::config('system.file')->get('default_scheme')."://celum_connect/".$id."_".$version."_".$dlf.".".$fileExtension,
          TRUE, FILE_EXISTS_REPLACE)){
          system_retrieve_file(
            $thumb_url,
            \Drupal::config('system.file')->get('default_scheme')."://celum_connect/".$id."_".$version."_".$dlf.".".$fileExtension."_thumb",
            TRUE,
            FILE_EXISTS_REPLACE);
          \Drupal::messenger()->addMessage("File uploaded successfully");
        }else{
          \Drupal::messenger()->addMessage("Failed to upload File",'error');
        }
      }
    }
  }

  function decrypt($sData){
    $secretKey = "ZbMchtd9DivzjPDi5QIio1iVERFnNZiSE33QKY3Gw9rYfCNLFiKloJQt3zi4";
    $sResult = '';
    $sData   = $this->decode_base64($sData);
    for($i=0;$i<strlen($sData);$i++){
      $sChar    = substr($sData, $i, 1);
      $sKeyChar = substr($secretKey, ($i % strlen($secretKey)) - 1, 1);
      $sChar    = chr(ord($sChar) - ord($sKeyChar));
      $sResult .= $sChar;
    }
    return $sResult;
  }

  function decode_base64($sData){
    $sBase64 = strtr($sData, '-_', '+/');
    return base64_decode($sBase64.'==');
  }

}
